package com.jeeplus.test.cw.projectReport.controller;

import com.jeeplus.aop.logging.annotation.ApiLog;
import com.jeeplus.common.utils.ResponseUtil;
import com.jeeplus.sys.constant.enums.LogTypeEnum;
import com.jeeplus.test.cw.projectReport.domain.CwProjectReportData;
import com.jeeplus.test.cw.projectReport.service.CwProjectReportReviewService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

/**
 * @author: 王强
 * @create: 2022-12-12 14:45
 **/
@RestController
@Api(tags ="报告审核校对")
@RequestMapping(value = "/projectReport/review")
public class CwProjectReportReviewController {

    @Resource
    private CwProjectReportReviewService service;

    /**
     * 保存财务报告信息-流程
     * @param reportData
     * @return
     */
    @ApiLog(value = "保存财务报告信息-流程", type = LogTypeEnum.SAVE)
    @PreAuthorize("hasAnyAuthority('cwProjectReportReview:add','cwProjectReportReview:edit')")
    @PostMapping("saveForm")
    public ResponseEntity saveForm(@Valid @RequestBody CwProjectReportData reportData) throws Exception {
        String id = service.saveForm(reportData);
        return ResponseUtil.newInstance().add("businessTable", "cw_project_report_review").add("businessId", id).ok ("操作成功");
    }

    /**
     * 根据id修改状态值status
     * @param data
     * @return
     */
    @ApiOperation(value = "根据id修改状态值status")
    @PostMapping(value = "/updateReviewStatysByReportId")
    public ResponseEntity<String> updateReviewStatysByReportId(@RequestBody CwProjectReportData data) {
        String s = service.updateReviewStatysByReportId(data);
        return ResponseEntity.ok(s);
    }

    /**
     * 查询客户信息数据
     * @param id
     * @return
     */
    @ApiLog("查询财务报告数据")
    @PreAuthorize ("hasAnyAuthority('cwProjectReport:view','cwProjectReport:add','cwProjectReport:edit')")
    @GetMapping("queryByReportId")
    public ResponseEntity queryByReportId(@RequestParam("id") String id) throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        CwProjectReportData projectReportData = service.queryByReportId(id);
        return ResponseEntity.ok (projectReportData);
    }

    /**
     * 查询客户信息数据
     * @param procInsId
     * @return
     */
    @ApiLog("查询财务报告数据")
    @PreAuthorize ("hasAnyAuthority('cwProjectReport:view','cwProjectReport:add','cwProjectReport:edit')")
    @GetMapping("getProcessStatus")
    public ResponseEntity getProcessStatus(@RequestParam("procInsId") String procInsId) throws IllegalAccessException, NoSuchMethodException, InvocationTargetException {
        String taskName = service.getProcessStatus(procInsId);
        return ResponseEntity.ok (taskName);
    }
}
