/**
 * Copyright &copy; 2021-2026 <a href="http://www.jeeplus.org/">JeePlus</a> All rights reserved.
 */
package com.jeeplus.sys.mapper;

import com.baomidou.mybatisplus.annotation.InterceptorIgnore;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jeeplus.sys.service.dto.OfficeDTO;
import com.jeeplus.sys.service.dto.UserDTO;
import com.jeeplus.sys.domain.User;
import org.apache.ibatis.annotations.Delete;
import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 用户MAPPER接口
 *
 * @author jeeplus
 * @version 2021-05-16
 */
public interface UserMapper extends BaseMapper<User> {


    /**
     * 根据条件查询用户
     *
     * @param queryWrapper
     * @return
     */
    @InterceptorIgnore(dataPermission = "true")
    UserDTO get(@Param(Constants.WRAPPER) QueryWrapper queryWrapper);

    /**
     * 根据条件查询用户id
     * @param loginName
     * @return
     */
    String getUserIdByLogin(@Param("loginName") String loginName);

    /**
     * 根据条件查询用户id
     * @param name
     * @return
     */
    String getUserIdByName(@Param("name") String name);

    /**
     * 获取用户列表
     *
     * @param queryWrapper
     * @return
     */
    IPage <UserDTO> findList(Page <UserDTO> page, @Param(Constants.WRAPPER) QueryWrapper queryWrapper);

    /**
     * 按角色获取用户列表
     *
     * @param queryWrapper
     * @return
     */
    IPage <UserDTO> findListByRole(Page <UserDTO> page, @Param(Constants.WRAPPER) QueryWrapper queryWrapper);


    /**
     * 按岗位获取用户列表
     *
     * @param postId
     * @return
     */
    List <UserDTO> findListByPostId(String postId);



    /**
     * 删除用户角色关联数据
     *
     * @param userId
     * @return
     */
    void deleteUserRole(String userId);

    /**
     * 插入用户角色关联数据
     * @param userId
     * @param roleId
     * @return
     */
    void insertUserRole(String userId, String roleId);

    /**
     * 删除用户岗位关联数据
     *
     * @param userId
     * @return
     */
    void deleteUserPost(String userId);

    /**
     * 插入用户角色关联数据
     * @param userId
     * @param postId
     * @return
     */
    void insertUserPost(String userId, String postId);


    /**
     * 按公司获取用户列表
     *
     * @param companyId
     * @return
     */
    List <OfficeDTO> findListByCompanyId(@Param("companyId") String companyId, @Param("type") String type);

    /**
     * 根据部门获取员工数据
     * @param officeIds
     * @param type
     * @return
     */
    List <OfficeDTO> getUserByOffice(@Param("officeIds") List<String> officeIds,@Param("type") String type);

    /**
     * 根据部门id查询部门下所有有效用户信息
     * @param officeIds
     * @return
     */
    List<UserDTO> getUserByOfficeIds(@Param("officeIds") List<String> officeIds);

    /**
     * getUserIdByLoginOrMobile
     * @param userName
     * @return
     */
    List<UserDTO> getUserIdByLoginOrMobile(String userName);

    /**
     * 根据用户手机号和当前人id查询非当前登录人的信息
     *
     * @param userDTO
     * @return
     */
    List <UserDTO> getUserInfoByMobile(UserDTO userDTO);

    /**
     * 根据角色名称查询角色下人员信息
     *
     * @param roleName 角色名称
     * @return
     */
    List <UserDTO> getUserInfoByRoleName(String roleName);

    /**
     * 根据人员名称查询人员信息
     *
     * @param name 名称
     * @return
     */
    UserDTO getUserByName(String name);

}
